const Express = require("express")();
const Http = require("http").Server(Express);
const Socketio = require("socket.io")(Http, {
    cors: {
        origin: '*',
    }
});

Http.listen(3000, () => {
    console.log("Listening at :3000...");
});

var currentCard = {};
var clients = {}; // Value = ClientID / Key = [SessionID, Team] / Team = 0 (total), 1 (red) or 2 (blue)
var clientsInSession = {}; // Value = SessionID / Key = [total, red, blue}

function getSumClients (sessionName, team, action) { // Team = 0 (total), 1 (red) or 2 (blue) / Action = 1 (add) or -1 (remove)
    if (!clientsInSession.hasOwnProperty(sessionName)) {
        clientsInSession[sessionName] = [0, 0, 0];
    }
    if (team === 0) {
        clientsInSession[sessionName] = [clientsInSession[sessionName][0] + action, clientsInSession[sessionName][1], clientsInSession[sessionName][2]];
    }
    if (team === 1) {
        clientsInSession[sessionName] = [clientsInSession[sessionName][0] + action, clientsInSession[sessionName][1] + action, clientsInSession[sessionName][2]];
    }
    if (team === 2) {
        clientsInSession[sessionName] = [clientsInSession[sessionName][0] + action, clientsInSession[sessionName][1], clientsInSession[sessionName][2] + action];
    }
    return {total: clientsInSession[sessionName][0], red: clientsInSession[sessionName][1], blue: clientsInSession[sessionName][2]};
}

Socketio.on("connection", socket => {
    console.log('new Socket connect');
    clients[socket.id] = [null, null];

    for (var key in currentCard) {
        socket.emit(key, currentCard[key])
    }
    socket.on("newGame", data => {
        console.log('New Game from: ', data.sessionName, ':newGame');
        Socketio.emit(data.sessionName+":newGame", true);
        socket.emit(data.sessionName+":newGame", true);
    });
    socket.on("newCard", data => {
        console.log('New Card from: ', data.sessionName, ':newCard');
        currentCard[data.sessionName+":newCard"] = data.cardID;
        Socketio.emit(data.sessionName+":newCard", data.cardID)
    });
    socket.on("newRound", data => {
        console.log('New Round from: ', data.isOpen);
        Socketio.emit(data.sessionName+":endRound", false);
        socket.emit(data.sessionName+":endRound", false);
        Socketio.emit(data.sessionName+":newRound", true);
        socket.emit(data.sessionName+":newRound", true);
        Socketio.emit(data.sessionName+":newGame", false);
        socket.emit(data.sessionName+":newGame", false);
    });
    socket.on("endRound", data => {
        console.log('End Round from: ', data.isOpen);
        Socketio.emit(data.sessionName+":newRound", false);
        socket.emit(data.sessionName+":newRound", false);
        Socketio.emit(data.sessionName+":endRound", true);
        socket.emit(data.sessionName+":endRound", true);
        let sumClients = getSumClients(data.sessionName, null, 0);
        Socketio.emit(data.sessionName+":sessionPlayers", sumClients);
    });
    socket.on("canPressTabooh", data => {
        console.log('CanPressTabooh from: ', data.sessionName, data.canPressTabooh);
        Socketio.emit(data.sessionName+":canPressTabooh", data.canPressTabooh);
        socket.emit(data.sessionName+":canPressTabooh", data.canPressTabooh);
    });
    socket.on("historyChanged", data => {
        console.log('historyChanged from: ', data.sessionName);
        Socketio.emit(data.sessionName+":historyChanged", data);
        socket.emit(data.sessionName+":historyChanged", data);
    });
    socket.on("sessionPlayers", data => {
        const client = clients[socket.id];
        console.log('CLIENT: ', client, data.team);
        if (data.sessionName === null && client[0] !== null) {
            let sumClients = getSumClients(client[0], client[1], -1);
            Socketio.emit(client[0]+":sessionPlayers", sumClients);
            clients[socket.id] = [null, null];
        }
        else {
            if (data.sessionName !== client[0] || data.team !== client[1]) {
                let sumClients = getSumClients(client[0], client[1], -1);
                Socketio.emit(client[0]+":sessionPlayers", sumClients);
                clients[socket.id] = [data.sessionName, data.team];
                sumClients = getSumClients(data.sessionName, data.team, data.newAction);
                Socketio.emit(data.sessionName+":sessionPlayers", sumClients);
                socket.emit(data.sessionName+":sessionPlayers", sumClients);
            }
       }
    });
    socket.on("disconnect", () => {
        const client = clients[socket.id];
        console.log("DISCONNECT CLIENT: ", client);
        if (client !== null && client[0] !== null) {
            const sumClients = getSumClients(client[0], client[1], -1);
            Socketio.emit(client[0]+":sessionPlayers", sumClients);
        }
        delete clients[socket.id];
    });
});

